<?php
/*
 * tweeper - a Twitter to RSS web scraper
 * 
 * Copyright (C) 2013  Antonio Ospite <ospite@studenti.unina.it>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

date_default_timezone_set('UTC');

define('USER_AGENT', 'Mozilla/5.0 (Windows NT 6.1; rv:22.0) Gecko/20130405 Firefox/22.0');

function str_to_gmdate($date)
{
  $timestamp = strtotime($date);
  return epoch_to_gmdate($timestamp);
}

function epoch_to_gmdate($timestamp)
{
  return gmdate('D, d M Y H:i:s', $timestamp) . ' GMT';
}

function get_url_info($url)
{
  $ch = curl_init($url);
  curl_setopt_array($ch, array(
      CURLOPT_HEADER => TRUE,
      CURLOPT_NOBODY => TRUE,
      CURLOPT_FOLLOWLOCATION => TRUE,  // follow http redirects to get the real URL
      CURLOPT_RETURNTRANSFER => TRUE,
      CURLOPT_SSL_VERIFYHOST => FALSE,
      CURLOPT_SSL_VERIFYPEER => FALSE,
      CURLOPT_USERAGENT => USER_AGENT,
  ));
  curl_exec($ch);
  $url_info = curl_getinfo($ch);;
  curl_close($ch);

  return $url_info;
}

function generate_enclosure($url)
{
  $url_info = get_url_info($url);

  switch($url_info['content_type']) {
    case "video/avi":
    case "image/gif":
    case "image/jpeg":
    case "audio/x-midi":
    case "audio/mpeg":
    case "video/mpeg":
    case "audio/vorbis":
    case "application/ogg":
    case "audio/webm":
    case "audio/aac":
    case "audio/mp4":
    case "audio/wav":
    case "video/mp4":
    case "video/ogg":
      $dom = new DomDocument();
      $enc = $dom->createElement('enclosure');
      $enc->setAttribute( 'url', $url_info['url']);
      $enc->setAttribute( 'length', $url_info['download_content_length']);
      $enc->setAttribute( 'type', $url_info['content_type']);

      $dom->appendChild($enc);

      return $dom->saveXML($enc);
    default:
        return;
  }
}


class Tweeper {

  public function __construct($stylesheet) {
    $stylesheet_contents = $this->get_contents($stylesheet);

    $xslDoc = new DOMDocument();
    $xslDoc->loadXML($stylesheet_contents);

    $this->xsltProcessor = new XSLTProcessor();
    $this->xsltProcessor->registerPHPFunctions();
    $this->xsltProcessor->importStylesheet($xslDoc);
  }

  private function get_contents($uri) {
    # https://www.wjsams.com/c/docs/Wiki/Php-HowToSetUserAgentOnFileGetContents
    $opts = array(
      'http' => array(
        'method' => "GET",
        'header' => join(array(
          "Accept-language: en\r\n",
          "User-Agent: {" + USER_AGENT + "}\r\n"
        ))
      )
    );

    $context = stream_context_create($opts);
    $contents = file_get_contents($uri, false, $context);
    return $contents;
  }

  public function tweep($uri) {
    $html = $this->get_contents($uri);

    $xmlDoc = new DOMDocument();
    $xmlDoc->loadHTML($html);

    $output = $this->xsltProcessor->transformToXML($xmlDoc);

    if (FALSE === $output) {
      trigger_error('XSL transformation failed.', E_USER_ERROR);
      return NULL;
    }
    return $output;
  }
}

function usage($argv)
{
  if (php_sapi_name() != 'cli')
    $usage = htmlentities("{$_SERVER['SCRIPT_NAME']}?src_url=<src_url>");
  else
    $usage = "{$argv[0]} <src_url>\n";

  return "usage: $usage";
}

if (isset($_GET['src_url'])) {
  $src_url = $_GET['src_url'];
} else if (isset($argv[1])) {
  if ($argv[1] == "-h" || $argv[1] == "--help") {
    echo usage($argv);
    return 0;
  }
  $src_url = $argv[1];
} else {
  die(usage($argv));
}

$url = parse_url($src_url);
if (FALSE === $url || empty($url["host"]))
  die("Invalid url: $src_url\n");

$stylesheet = __DIR__ . "/rss_converter_" . $url["host"] . ".xsl";
if (FALSE === file_exists($stylesheet))
  die("Conversion to RSS not supported: {$url["host"]}\n");

$tweeper = new Tweeper($stylesheet);
echo $tweeper->tweep($src_url);
